==> dcl-c_5.3-1.tar.gz: dcl-c-5.3/src/grph2/uhpack/uhdifz.c <==
/* src/grph2/uhpack/uhdifz.f -- translated by f2c (version 20020621).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)
*/

#include "libtinyf2c.h"

/* Common Block Declarations */

struct {
    integer irmode, irmodr;
} szbtn2_;

#define szbtn2_1 szbtn2_

struct {
    logical lclip;
} szbtn3_;

#define szbtn3_1 szbtn3_

/* Table of constant values */

static real c_b15 = 0.f;
static real c_b18 = 1.f;
static integer c__1 = 1;

/* ----------------------------------------------------------------------- */
/*     Copyright (C) 2000-2004 GFD Dennou Club. All rights reserved. */
/* ----------------------------------------------------------------------- */
/* Subroutine */ int uhdifz_(integer *n, real *upx1, real *upx2, real *upy, 
	integer *itpat1, integer *itpat2)
{
    /* System generated locals */
    integer i__1;
    real r__1;

    /* Builtin functions */
    integer s_wsfi(icilist *), do_fio(integer *, char *, ftnlen), e_wsfi(void)
	    ;

    /* Local variables */
    static integer i__, ir;
    static real dy, rx0, ry0, rx1, ry1, rx2, ry2, dxn, dxp, vxc, vyc, rot, 
	    uyn, vyn, uyp, vyp;
    static logical lxc1, lxc2;
    static real ux1n, ux2n, ux1p, ux2p, vx1p, vx2p, vx1n, vx2n;
    static char cobj[80];
    static real uref;
    extern /* Subroutine */ int cdblk_(char *, ftnlen);
    static logical lmiss;
    static real rmiss;
    static logical lyuni;
    static real uymin, uymax;
    extern /* Subroutine */ int stfpr2_(real *, real *, real *, real *), 
	    gllget_(char *, logical *, ftnlen);
    static real rundef;
    extern /* Subroutine */ int glrget_(char *, real *, ftnlen), sglget_(char 
	    *, logical *, ftnlen), msgdmp_(char *, char *, char *, ftnlen, 
	    ftnlen, ftnlen), sgrget_(char *, real *, ftnlen), stftrf_(real *, 
	    real *, real *, real *), swocls_(char *, ftnlen), uurget_(char *, 
	    real *, ftnlen), uuqidv_(real *, real *), swoopn_(char *, char *, 
	    ftnlen, ftnlen), szcltv_(void), szstni_(integer *), szoptv_(void),
	     szsttv_(real *, real *);

    /* Fortran I/O blocks */
    static icilist io___13 = { 0, cobj, 0, "(2I8)", 80, 1 };


    /* Parameter adjustments */
    --upy;
    --upx2;
    --upx1;

    /* Function Body */
    if (*n < 1) {
	msgdmp_("E", "UHDIFZ", "NUMBER OF POINTS IS LESS THAN 1.", (ftnlen)1, 
		(ftnlen)6, (ftnlen)32);
    }
    if (*itpat1 == 0 || *itpat2 == 0) {
	msgdmp_("M", "UHDIFZ", "TONE PAT. INDEX IS 0 / DO NOTHING.", (ftnlen)
		1, (ftnlen)6, (ftnlen)34);
	return 0;
    }
    if (*itpat1 < 0 || *itpat2 < 0) {
	msgdmp_("E", "UHDIFZ", "TONE PAT. INDEX IS LESS THAN 0.", (ftnlen)1, (
		ftnlen)6, (ftnlen)31);
    }
    sglget_("LCLIP", &szbtn3_1.lclip, (ftnlen)5);
    glrget_("RUNDEF", &rundef, (ftnlen)6);
    glrget_("RMISS", &rmiss, (ftnlen)5);
    gllget_("LMISS", &lmiss, (ftnlen)5);
    stfpr2_(&c_b15, &c_b15, &rx0, &ry0);
    stfpr2_(&c_b15, &c_b18, &rx1, &ry1);
    stfpr2_(&c_b18, &c_b15, &rx2, &ry2);
    rot = (rx2 - rx0) * (ry1 - ry0) - (ry2 - ry0) * (rx1 - rx0);
    if (rot > 0.f) {
	ir = 0;
    } else {
	ir = 1;
    }
    s_wsfi(&io___13);
    do_fio(&c__1, (char *)&(*itpat1), (ftnlen)sizeof(integer));
    do_fio(&c__1, (char *)&(*itpat2), (ftnlen)sizeof(integer));
    e_wsfi();
    cdblk_(cobj, (ftnlen)80);
    swoopn_("UHDIFZ", cobj, (ftnlen)6, (ftnlen)80);
    lyuni = upy[1] == rundef;
    lxc1 = upx1[1] == rundef;
    lxc2 = upx2[1] == rundef;
    if (lyuni) {
	uuqidv_(&uymin, &uymax);
	if (uymin == rundef) {
	    sgrget_("UYMIN", &uymin, (ftnlen)5);
	}
	if (uymax == rundef) {
	    sgrget_("UYMAX", &uymax, (ftnlen)5);
	}
	dy = (uymax - uymin) / (*n - 1);
    }
    if (lxc1 || lxc2) {
	uurget_("UREF", &uref, (ftnlen)4);
    }
    i__1 = *n - 1;
    for (i__ = 1; i__ <= i__1; ++i__) {
	if (lyuni) {
	    uyp = uymin + dy * (i__ - 1);
	    uyn = uymin + dy * i__;
	} else {
	    uyp = upy[i__];
	    uyn = upy[i__ + 1];
	}
	if (lxc1) {
	    ux1p = uref;
	    ux1n = uref;
	} else {
	    ux1p = upx1[i__];
	    ux1n = upx1[i__ + 1];
	}
	if (lxc2) {
	    ux2p = uref;
	    ux2n = uref;
	} else {
	    ux2p = upx2[i__];
	    ux2n = upx2[i__ + 1];
	}
	if (! ((uyp == rmiss || uyn == rmiss || ux1p == rmiss || ux1n == 
		rmiss || ux2p == rmiss || ux2n == rmiss) && lmiss)) {
	    stftrf_(&ux1p, &uyp, &vx1p, &vyp);
	    stftrf_(&ux2p, &uyp, &vx2p, &vyp);
	    stftrf_(&ux1n, &uyn, &vx1n, &vyn);
	    stftrf_(&ux2n, &uyn, &vx2n, &vyn);
	    if ((vx2p - vx1p) * (vx2n - vx1n) >= 0.f) {
		if (vx2p >= vx1p) {
		    szbtn2_1.irmode = 0;
		} else {
		    szbtn2_1.irmode = 1;
		}
		szbtn2_1.irmodr = (szbtn2_1.irmode + ir) % 2;
		if (ux2p > ux1p) {
		    szstni_(itpat1);
		} else {
		    szstni_(itpat2);
		}
		szoptv_();
		szsttv_(&vx1p, &vyp);
		szsttv_(&vx2p, &vyp);
		szsttv_(&vx2n, &vyn);
		szsttv_(&vx1n, &vyn);
		szsttv_(&vx1p, &vyp);
		szcltv_();
	    } else {
		dxp = (r__1 = vx2p - vx1p, dabs(r__1));
		dxn = (r__1 = vx2n - vx1n, dabs(r__1));
		vyc = (vyp * dxn + vyn * dxp) / (dxp + dxn);
		vxc = (vx1p * dxn + vx1n * dxp) / (dxp + dxn);
		if (vx2p >= vx1p) {
		    szbtn2_1.irmode = 0;
		} else {
		    szbtn2_1.irmode = 1;
		}
		szbtn2_1.irmodr = (szbtn2_1.irmode + ir) % 2;
		if (ux2p > ux1p) {
		    szstni_(itpat1);
		} else {
		    szstni_(itpat2);
		}
		szoptv_();
		szsttv_(&vx1p, &vyp);
		szsttv_(&vx2p, &vyp);
		szsttv_(&vxc, &vyc);
		szsttv_(&vx1p, &vyp);
		szcltv_();
		szbtn2_1.irmode = (szbtn2_1.irmode + 1) % 2;
		szbtn2_1.irmodr = (szbtn2_1.irmode + ir) % 2;
		if (ux2n > ux1n) {
		    szstni_(itpat1);
		} else {
		    szstni_(itpat2);
		}
		szoptv_();
		szsttv_(&vx2n, &vyn);
		szsttv_(&vx1n, &vyn);
		szsttv_(&vxc, &vyc);
		szsttv_(&vx2n, &vyn);
		szcltv_();
	    }
	}
/* L20: */
    }
    swocls_("UHDIFZ", (ftnlen)6);
    return 0;
} /* uhdifz_ */

==> dcl-c_5.3.3.orig.tar.gz: dcl-c-5.3.3/src/grph2/uhpack/uhdifz.c <==
/*  -- translated by f2c (version 20050501).
   You must link the resulting object file with libf2c:
	on Microsoft Windows system, link with libf2c.lib;
	on Linux or Unix systems, link with .../path/to/libf2c.a -lm
	or, if you install libf2c.a in a standard place, with -lf2c -lm
	-- in that order, at the end of the command line, as in
		cc *.o -lf2c -lm
	Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,

		http://www.netlib.org/f2c/libf2c.zip
*/

#include "libtinyf2c.h"

/* Common Block Declarations */

struct {
    integer irmode, irmodr;
} szbtn2_;

#define szbtn2_1 szbtn2_

struct {
    logical lclip;
} szbtn3_;

#define szbtn3_1 szbtn3_

/* Table of constant values */

static real c_b15 = 0.f;
static real c_b18 = 1.f;
static integer c__1 = 1;

/* ----------------------------------------------------------------------- */
/*     Copyright (C) 2000-2004 GFD Dennou Club. All rights reserved. */
/* ----------------------------------------------------------------------- */
/* Subroutine */ int uhdifz_(integer *n, real *upx1, real *upx2, real *upy, 
	integer *itpat1, integer *itpat2)
{
    /* System generated locals */
    integer i__1;
    real r__1;

    /* Builtin functions */
    integer s_wsfi(icilist *), do_fio(integer *, char *, ftnlen), e_wsfi(void)
	    ;

    /* Local variables */
    static integer i__, ir;
    static real dy, rx0, ry0, rx1, ry1, rx2, ry2, dxn, dxp, vxc, vyc, rot, 
	    uyn, vyn, uyp, vyp;
    static logical lxc1, lxc2;
    static real ux1n, ux2n, ux1p, ux2p, vx1p, vx2p, vx1n, vx2n;
    static char cobj[80];
    static real uref;
    extern /* Subroutine */ int cdblk_(char *, ftnlen);
    static logical lmiss;
    static real rmiss;
    static logical lyuni;
    static real uymin, uymax;
    extern /* Subroutine */ int stfpr2_(real *, real *, real *, real *), 
	    gllget_(char *, logical *, ftnlen);
    static real rundef;
    extern /* Subroutine */ int glrget_(char *, real *, ftnlen), sglget_(char 
	    *, logical *, ftnlen), msgdmp_(char *, char *, char *, ftnlen, 
	    ftnlen, ftnlen), sgrget_(char *, real *, ftnlen), stftrf_(real *, 
	    real *, real *, real *), swocls_(char *, ftnlen), uurget_(char *, 
	    real *, ftnlen), uuqidv_(real *, real *), swoopn_(char *, char *, 
	    ftnlen, ftnlen), szcltv_(void), szstni_(integer *), szoptv_(void),
	     szsttv_(real *, real *);

    /* Fortran I/O blocks */
    static icilist io___13 = { 0, cobj, 0, "(2I8)", 80, 1 };


    /* Parameter adjustments */
    --upy;
    --upx2;
    --upx1;

    /* Function Body */
    if (*n < 1) {
	msgdmp_("E", "UHDIFZ", "NUMBER OF POINTS IS LESS THAN 1.", (ftnlen)1, 
		(ftnlen)6, (ftnlen)32);
    }
    if (*itpat1 == 0 || *itpat2 == 0) {
	msgdmp_("M", "UHDIFZ", "TONE PAT. INDEX IS 0 / DO NOTHING.", (ftnlen)
		1, (ftnlen)6, (ftnlen)34);
	return 0;
    }
    if (*itpat1 < 0 || *itpat2 < 0) {
	msgdmp_("E", "UHDIFZ", "TONE PAT. INDEX IS LESS THAN 0.", (ftnlen)1, (
		ftnlen)6, (ftnlen)31);
    }
    sglget_("LCLIP", &szbtn3_1.lclip, (ftnlen)5);
    glrget_("RUNDEF", &rundef, (ftnlen)6);
    glrget_("RMISS", &rmiss, (ftnlen)5);
    gllget_("LMISS", &lmiss, (ftnlen)5);
    stfpr2_(&c_b15, &c_b15, &rx0, &ry0);
    stfpr2_(&c_b15, &c_b18, &rx1, &ry1);
    stfpr2_(&c_b18, &c_b15, &rx2, &ry2);
    rot = (rx2 - rx0) * (ry1 - ry0) - (ry2 - ry0) * (rx1 - rx0);
    if (rot > 0.f) {
	ir = 0;
    } else {
	ir = 1;
    }
    s_wsfi(&io___13);
    do_fio(&c__1, (char *)&(*itpat1), (ftnlen)sizeof(integer));
    do_fio(&c__1, (char *)&(*itpat2), (ftnlen)sizeof(integer));
    e_wsfi();
    cdblk_(cobj, (ftnlen)80);
    swoopn_("UHDIFZ", cobj, (ftnlen)6, (ftnlen)80);
    lyuni = upy[1] == rundef;
    lxc1 = upx1[1] == rundef;
    lxc2 = upx2[1] == rundef;
    if (lyuni) {
	uuqidv_(&uymin, &uymax);
	if (uymin == rundef) {
	    sgrget_("UYMIN", &uymin, (ftnlen)5);
	}
	if (uymax == rundef) {
	    sgrget_("UYMAX", &uymax, (ftnlen)5);
	}
	dy = (uymax - uymin) / (*n - 1);
    }
    if (lxc1 || lxc2) {
	uurget_("UREF", &uref, (ftnlen)4);
    }
    i__1 = *n - 1;
    for (i__ = 1; i__ <= i__1; ++i__) {
	if (lyuni) {
	    uyp = uymin + dy * (i__ - 1);
	    uyn = uymin + dy * i__;
	} else {
	    uyp = upy[i__];
	    uyn = upy[i__ + 1];
	}
	if (lxc1) {
	    ux1p = uref;
	    ux1n = uref;
	} else {
	    ux1p = upx1[i__];
	    ux1n = upx1[i__ + 1];
	}
	if (lxc2) {
	    ux2p = uref;
	    ux2n = uref;
	} else {
	    ux2p = upx2[i__];
	    ux2n = upx2[i__ + 1];
	}
	if (! ((uyp == rmiss || uyn == rmiss || ux1p == rmiss || ux1n == 
		rmiss || ux2p == rmiss || ux2n == rmiss) && lmiss)) {
	    stftrf_(&ux1p, &uyp, &vx1p, &vyp);
	    stftrf_(&ux2p, &uyp, &vx2p, &vyp);
	    stftrf_(&ux1n, &uyn, &vx1n, &vyn);
	    stftrf_(&ux2n, &uyn, &vx2n, &vyn);
	    if ((vx2p - vx1p) * (vx2n - vx1n) >= 0.f) {
		if (vx2p >= vx1p) {
		    szbtn2_1.irmode = 0;
		} else {
		    szbtn2_1.irmode = 1;
		}
		szbtn2_1.irmodr = (szbtn2_1.irmode + ir) % 2;
		if (ux2p > ux1p) {
		    szstni_(itpat1);
		} else {
		    szstni_(itpat2);
		}
		szoptv_();
		szsttv_(&vx1p, &vyp);
		szsttv_(&vx2p, &vyp);
		szsttv_(&vx2n, &vyn);
		szsttv_(&vx1n, &vyn);
		szsttv_(&vx1p, &vyp);
		szcltv_();
	    } else {
		dxp = (r__1 = vx2p - vx1p, abs(r__1));
		dxn = (r__1 = vx2n - vx1n, abs(r__1));
		vyc = (vyp * dxn + vyn * dxp) / (dxp + dxn);
		vxc = (vx1p * dxn + vx1n * dxp) / (dxp + dxn);
		if (vx2p >= vx1p) {
		    szbtn2_1.irmode = 0;
		} else {
		    szbtn2_1.irmode = 1;
		}
		szbtn2_1.irmodr = (szbtn2_1.irmode + ir) % 2;
		if (ux2p > ux1p) {
		    szstni_(itpat1);
		} else {
		    szstni_(itpat2);
		}
		szoptv_();
		szsttv_(&vx1p, &vyp);
		szsttv_(&vx2p, &vyp);
		szsttv_(&vxc, &vyc);
		szsttv_(&vx1p, &vyp);
		szcltv_();
		szbtn2_1.irmode = (szbtn2_1.irmode + 1) % 2;
		szbtn2_1.irmodr = (szbtn2_1.irmode + ir) % 2;
		if (ux2n > ux1n) {
		    szstni_(itpat1);
		} else {
		    szstni_(itpat2);
		}
		szoptv_();
		szsttv_(&vx2n, &vyn);
		szsttv_(&vx1n, &vyn);
		szsttv_(&vxc, &vyc);
		szsttv_(&vx2n, &vyn);
		szcltv_();
	    }
	}
/* L20: */
    }
    swocls_("UHDIFZ", (ftnlen)6);
    return 0;
} /* uhdifz_ */
